#!/usr/bin/env python

"""
Simple built-in classes and functions.

Copyright (C) 2005, 2006 Paul Boddie <paul@boddie.org.uk>

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; see the file LICENCE.txt
If not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""

class object:
    def __bool__(self):
        # NOTE: Should really test for __len__ and then resort to False.
        # NOTE: See the reference: 3.3.1 Basic customization
        return False

class bool:
    def __bool__(self):
        return self

    def __str__(self):
        if self:
            return "True"
        else:
            return "False"

class int:
    def __iadd__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __isub__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __add__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __radd__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __sub__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __rsub__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __mul__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __rmul__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __div__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __rdiv__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __floordiv__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __rfloordiv__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __pow__(self, other):
        if isinstance(other, int):
            return int()
        else:
            raise TypeError

    def __lt__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            raise TypeError

    def __gt__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            raise TypeError

    def __le__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            raise TypeError

    def __ge__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            raise TypeError

    def __eq__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            raise TypeError

    def __ne__(self, other):
        if isinstance(other, int):
            return bool()
        else:
            raise TypeError

    def __neg__(self):
        return int()

    def __pos__(self):
        return self

    def __str__(self):
        return string()

    def __bool__(self):
        return self != 0

class long:
    def __iadd__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __isub__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __add__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __radd__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __sub__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __rsub__(self, other):
        if isinstance(other, int):
            return long()
        elif isinstance(other, long):
            return long()
        else:
            raise TypeError

    def __lt__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            raise TypeError

    def __gt__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            raise TypeError

    def __le__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            raise TypeError

    def __ge__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            raise TypeError

    def __eq__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            raise TypeError

    def __ne__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        else:
            raise TypeError

    def __neg__(self):
        return long()

    def __pos__(self):
        return self

    def __str__(self):
        return string()

    def __bool__(self):
        return self != 0

class float:
    def __iadd__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __isub__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __add__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __radd__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __sub__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __rsub__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __mul__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __rmul__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __div__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __rdiv__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __floordiv__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __rfloordiv__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __pow__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __rpow__(self, other):
        if isinstance(other, int):
            return float()
        elif isinstance(other, long):
            return float()
        elif isinstance(other, float):
            return float()
        else:
            raise TypeError

    def __lt__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            raise TypeError

    def __gt__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            raise TypeError

    def __le__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            raise TypeError

    def __ge__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            raise TypeError

    def __eq__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            raise TypeError

    def __ne__(self, other):
        if isinstance(other, int):
            return bool()
        elif isinstance(other, long):
            return bool()
        elif isinstance(other, float):
            return bool()
        else:
            raise TypeError

    def __neg__(self):
        return float()

    def __pos__(self):
        return self

    def __str__(self):
        return string()

    def __bool__(self):
        return self != 0

class str:
    def __init__(self, x=None):
        x.__str__()

    def __add__(self, other):
        if isinstance(other, string):
            return string()
        else:
            raise TypeError

    def __radd__(self, other):
        if isinstance(other, string):
            return string()
        else:
            raise TypeError

    def __len__(self):
        return int()

    def __str__(self):
        return self

    def __bool__(self):
        return self.__len__() != 0

    def join(self, l):
        total = 0
        first = 1
        self_len = self.__len__()
        for i in l:
            if not first:
                total += self_len
            total += len(str(i))
            first = 0
        b = buffer(total)
        first = 1
        for i in l:
            if not first:
                b.append(self)
            b.append(str(i))
            first = 0
        s = str(b)
        return s

class buffer:
    def __init__(self, size):
        if not isinstance(size, int):
            raise TypeError

    def append(self, s):
        if not isinstance(s, string):
            raise TypeError
        pass

    def __str__(self):
        return string()

class list:
    def __init__(self, *args):
        self.next = None
        self.last = self
        for arg in args:
            self.append(arg)

    def __getitem__(self, index):
        i = 0
        n = self
        # NOTE: Support negative indices using last.
        while i < index and n.next is not None:
            n = n.next
            i += 1
        if n.next is not None:
            return n.value
        else:
            raise IndexError() # NOTE: Make this compliant with Python!

    def __setitem__(self, index, value):
        i = 0
        n = self
        # NOTE: Support negative indices using last.
        while i < index and n.next is not None:
            n = n.next
            i += 1
        if n.next is not None:
            n.value = value
        else:
            raise IndexError() # NOTE: Make this compliant with Python!

    def __getslice__(self, start, end=None):
        slice = []
        i = 0
        n = self
        # NOTE: Support negative indices using last.
        while (end is None or i < end) and n.next is not None:
            if i >= start:
                slice.append(n.value)
            n = n.next
            i += 1
        return slice

    def __setslice__(self, start, end, slice):
        i = 0
        n = self
        j = 0
        p = slice
        # NOTE: Support negative indices using last.
        # NOTE: Support appending when start >= len(self).
        while i < end and n is not None and p is not None:
            if i >= start:
                n.value = p.value
                p = p.next
                j += 1
            n = n.next
            i += 1

    def append(self, value):
        n = self.last
        n.value = value
        n.next = self.__class__()
        self.last = n.next

    def __len__(self):
        i = 0
        n = self
        while n.next is not None:
            n = n.next
            i += 1
        return i

    def __add__(self, other):
        result = self.__class__()
        for value in self:
            result.append(value)
        for value in other:
            result.append(value)
        return result

    def __iadd__(self, other):
        for value in other:
            self.append(value)
        return self

    def __str__(self):
        output = ["["]
        n = self
        first = 1
        while n.next is not None:
            if not first:
                output.append(", ")
            else:
                first = 0
            output.append(str(n.value))
            n = n.next
        output.append("]")
        return "".join(output)

    def __iter__(self):
        return listiterator(self)

    def __bool__(self):
        return self.__len__() != 0

class listiterator:
    def __init__(self, l):
        self.l = l

    def next(self):
        l = self.l
        next = l.next
        if next is not None:
            self.l = next
            return l.value
        else:
            raise StopIteration

    def __bool__(self):
        return bool()

class tuple:
    def __init__(self, *args):
        self.next = None
        self.last = self
        for arg in args:
            self.append(arg)

    def __getitem__(self, index):
        i = 0
        n = self
        # NOTE: Support negative indices using last.
        while i < index and n.next is not None:
            n = n.next
            i += 1
        if n.next is not None:
            return n.value
        else:
            raise IndexError() # NOTE: Make this compliant with Python!

    def __getslice__(self, start, end=None):
        # NOTE: Should probably return a tuple.
        slice = []
        i = 0
        n = self
        # NOTE: Support negative indices using last.
        while (end is None or i < end) and n.next is not None:
            if i >= start:
                slice.append(n.value)
            n = n.next
            i += 1
        return slice

    # NOTE: The append method should be internal at most.

    def append(self, value):
        n = self.last
        n.value = value
        n.next = self.__class__()
        self.last = n.next

    def __len__(self):
        i = 0
        n = self
        while n.next is not None:
            n = n.next
            i += 1
        return i

    def __add__(self, other):
        result = self.__class__()
        for value in self:
            result.append(value)
        for value in other:
            result.append(value)
        return result

    def __str__(self):
        output = ["("]
        n = self
        first = 1
        while n.next is not None:
            if not first:
                output.append(", ")
            else:
                first = 0
            output.append(str(n.value))
            n = n.next
        output.append(")")
        return "".join(output)

    def __iter__(self):
        return tupleiterator(self)

    def __bool__(self):
        return self.__len__() != 0

class tupleiterator:
    def __init__(self, l):
        self.l = l

    def next(self):
        l = self.l
        next = l.next
        if next is not None:
            self.l = next
            return l.value
        else:
            raise StopIteration

    def __bool__(self):
        return bool()

class dict:
    pass

class Exception:
    pass

class StopIteration(Exception):
    pass

class IndexError(Exception):
    pass

class AttributeError(Exception):
    pass

class TypeError(Exception):
    pass

class none:
    def __bool__(self):
        return False

    def __str__(self):
        return "None"

class undefined:
    pass

class xrange:
    def __init__(self, start, end, step=1):
        self.start = start
        self.end = end
        self.step = step
        self.current = start

    def __iter__(self):
        return self

    def next(self):
        if self.current >= self.end:
            raise StopIteration
        current = self.current
        self.current += self.step
        return current

# General functions.

def isinstance(obj, cls):
    return bool()

def issubclass(cls1, cls2):
    return bool()

def len(x):
    return x.__len__()

def max(*l):
    max_so_far = l[0]
    for i in l[1:]:
        if i > max_so_far:
            max_so_far = i
    return max_so_far

def range(start, end, step=None):
    if start == end:
        return []
    elif start > end:
        step = step or 1
        i = start
        result = []
        while i < end:
            result.append(i)
            i += 1
        return result
    else:
        step = step or -1
        i = start
        result = []
        while i > end:
            result.append(i)
            i -= 1
        return result

# Special values.

True = bool()
False = bool()
None = none()
Undefined = undefined()

# Special functions. These all operate on references at run-time.

def __is__(a, b):
    return bool()

def __is_not__(a, b):
    return bool()

def __not__(a):
    return bool()

# vim: tabstop=4 expandtab shiftwidth=4
